@@ -37,16 +37,10 @@ def newwconfig(vals=None):
     return wconfig.config(newrconfig(vals))
 
 # see wconfig.writefile()
-if pycompat.ispy3:
-    def written(c):
-        dest = pycompat.io.StringIO()
-        c.write(dest)
-        return hglib.fromunicode(dest.getvalue())
-else:
-    def written(c):
-        dest = util.bytesio()
-        c.write(dest)
-        return dest.getvalue()
+def written(c):
+    dest = pycompat.io.StringIO()
+    c.write(dest)
+    return hglib.fromunicode(dest.getvalue())
 
 def writetempfile(s):
     fd, path = tempfile.mkstemp(dir=_tempdir)
