@@ -5,10 +5,15 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+
 
 import os
 
+from typing import (
+    cast,
+    Optional,
+)
+
 from .qtcore import (
     QFile,
     QFileSystemWatcher,
@@ -55,7 +60,7 @@ from . import (
 def settingsfilename():
     """Return path to thg-reporegistry.xml as unicode"""
     s = QSettings()
-    dir = os.path.dirname(pycompat.unicode(s.fileName()))
+    dir = os.path.dirname(s.fileName())
     return dir + '/' + 'thg-reporegistry.xml'
 
 
@@ -81,12 +86,12 @@ class RepoTreeView(QTreeView):
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
         self.setAutoScroll(True)
-        self.setDragDropMode(QAbstractItemView.DragDrop)
-        self.setDefaultDropAction(Qt.MoveAction)
+        self.setDragDropMode(QAbstractItemView.DragDropMode.DragDrop)
+        self.setDefaultDropAction(Qt.DropAction.MoveAction)
         self.setDropIndicatorShown(True)
-        self.setEditTriggers(QAbstractItemView.DoubleClicked
-                             | QAbstractItemView.EditKeyPressed)
-        self.setSelectionBehavior(QAbstractItemView.SelectRows)
+        self.setEditTriggers(QAbstractItemView.EditTrigger.DoubleClicked
+                             | QAbstractItemView.EditTrigger.EditKeyPressed)
+        self.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
 
     def dragEnterEvent(self, event):
         if event.source() is self:
@@ -98,9 +103,9 @@ class RepoTreeView(QTreeView):
         for u in d.urls():
             root = paths.find_root(u.toLocalFile())
             if root:
-                event.setDropAction(Qt.LinkAction)
+                event.setDropAction(Qt.DropAction.LinkAction)
                 event.accept()
-                self.setState(QAbstractItemView.DraggingState)
+                self.setState(QAbstractItemView.State.DraggingState)
                 break
 
     def dropLocation(self, event):
@@ -115,7 +120,7 @@ class RepoTreeView(QTreeView):
             indicator = self.dropIndicatorPosition()
             group = index.parent()
             row = index.row()
-            if indicator == QAbstractItemView.BelowItem:
+            if indicator == QAbstractItemView.DropIndicatorPosition.BelowItem:
                 row = index.row() + 1
 
         return index, group, row
@@ -158,28 +163,28 @@ class RepoTreeView(QTreeView):
                 # Event is a drop of an external repo
                 accept = False
                 for u in data.urls():
-                    uroot = paths.find_root(pycompat.unicode(u.toLocalFile()))
+                    uroot = paths.find_root(u.toLocalFile())
                     if uroot and not m.isKnownRepoRoot(uroot, standalone=True):
                         repoindex = m.addRepo(uroot, row, group)
                         m.loadSubrepos(repoindex)
                         accept = True
                 if accept:
-                    event.setDropAction(Qt.LinkAction)
+                    event.setDropAction(Qt.DropAction.LinkAction)
                     event.accept()
                     self.dropAccepted.emit()
         self.setAutoScroll(False)
-        self.setState(QAbstractItemView.NoState)
+        self.setState(QAbstractItemView.State.NoState)
         self.viewport().update()
         self.setAutoScroll(True)
 
     def keyPressEvent(self, event):
-        if (event.key() in (Qt.Key_Enter, Qt.Key_Return)
-            and self.state() != QAbstractItemView.EditingState):
+        if (event.key() in (Qt.Key.Key_Enter, Qt.Key.Key_Return)
+            and self.state() != QAbstractItemView.State.EditingState):
             index = self.currentIndex()
             if index.isValid():
                 self.openRequested.emit(index)
                 return
-        if event.key() == Qt.Key_Delete:
+        if event.key() == Qt.Key.Key_Delete:
             index = self.currentIndex()
             if index.isValid():
                 self.removeRequested.emit(index)
@@ -195,7 +200,7 @@ class RepoTreeView(QTreeView):
             self.msg  = item.details()
         self.showMessage.emit(self.msg)
 
-        if event.buttons() == Qt.NoButton:
+        if event.buttons() == Qt.MouseButton.NoButton:
             # Bail out early to avoid tripping over this bug:
             # https://bugreports.qt.io/browse/QTBUG-10180
             return
@@ -215,7 +220,7 @@ class RepoTreeView(QTreeView):
 
     def sizeHint(self):
         size = super(RepoTreeView, self).sizeHint()
-        size.setWidth(QFontMetrics(self.font()).width('M') * 15)
+        size.setWidth(QFontMetrics(self.font()).horizontalAdvance('M') * 15)
         return size
 
 class RepoRegistryView(QDockWidget):
@@ -234,9 +239,9 @@ class RepoRegistryView(QDockWidget):
         self.watcher = None
         self._setupSettingActions()
 
-        self.setFeatures(QDockWidget.DockWidgetClosable |
-                         QDockWidget.DockWidgetMovable  |
-                         QDockWidget.DockWidgetFloatable)
+        self.setFeatures(QDockWidget.DockWidgetFeature.DockWidgetClosable |
+                         QDockWidget.DockWidgetFeature.DockWidgetMovable  |
+                         QDockWidget.DockWidgetFeature.DockWidgetFloatable)
         self.setWindowTitle(_('Repository Registry'))
 
         mainframe = QFrame()
@@ -252,7 +257,7 @@ class RepoRegistryView(QDockWidget):
         tv.setFirstColumnSpanned(0, QModelIndex(), True)
         tv.setColumnHidden(1, True)
 
-        tv.setContextMenuPolicy(Qt.CustomContextMenu)
+        tv.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
         tv.customContextMenuRequested.connect(self._onMenuRequested)
         tv.showMessage.connect(self.showMessage)
         tv.openRequested.connect(self._openRepoAt)
@@ -411,9 +416,8 @@ class RepoRegistryView(QDockWidget):
 
     # TODO: better to handle repositoryOpened signal by model
     @pyqtSlot(str)
-    def _addAndScanRepo(self, uroot):
+    def _addAndScanRepo(self, uroot: str):
         """Add repo if not exists; called when the workbench has opened it"""
-        uroot = pycompat.unicode(uroot)
         m = self.tview.model()
         assert m is not None
         knownindex = m.indexFromRepoRoot(uroot)
@@ -546,7 +550,7 @@ class RepoRegistryView(QDockWidget):
     ## Menu action handlers
     #
 
-    def _currentRepoRoot(self):
+    def _currentRepoRoot(self) -> Optional[str]:
         model = self.tview.model()
         index = self.tview.currentIndex()
         assert model is not None
@@ -579,159 +583,176 @@ class RepoRegistryView(QDockWidget):
         caption = _('Select repository directory to add')
         FD = QFileDialog
         path = FD.getExistingDirectory(caption=caption,
-                                       options=FD.ShowDirsOnly | FD.ReadOnly)
-        if path:
-            m = self.tview.model()
-            assert m is not None
-            uroot = paths.find_root(pycompat.unicode(path))
-            if uroot and not m.isKnownRepoRoot(uroot, standalone=True):
-                index = m.addRepo(uroot, parent=self.tview.currentIndex())
-                self._scanAddedRepo(index)
+                                       options=QFileDialog.Option.ShowDirsOnly | QFileDialog.Option.ReadOnly)
+        if not path:
+            # the used pressed "cancel"
+            return
+        m = self.tview.model()
+        assert m is not None
+        uroot = paths.find_root(path)
+        if not uroot:
+            qtlib.WarningMsgBox(
+                _('Repository not found'),
+                _('The selected directory (%s) does not appear to be '
+                  'a mercurial repository') % path,
+                parent=self)
+            return
+        if m.isKnownRepoRoot(uroot, standalone=True):
+            qtlib.InfoMsgBox(
+                _('Repository already in registry'),
+                _('The selected repository (%s) is already present in '
+                  'the registry. Not adding it again.') % uroot,
+                parent=self)
+            return
+        # this is a new mercurial repo, let's add it to the registry
+        index = m.addRepo(uroot, parent=self.tview.currentIndex())
+        self._scanAddedRepo(index)
 
     def addSubrepo(self):
         'menu action handler for adding a new subrepository'
         root = self._currentRepoRoot()
+        assert root is not None
+
         caption = _('Select an existing repository to add as a subrepo')
         FD = QFileDialog
-        path = pycompat.unicode(FD.getExistingDirectory(caption=caption,
-            directory=root, options=FD.ShowDirsOnly | FD.ReadOnly))
-        if path:
-            path = os.path.normpath(path)
-            sroot = paths.find_root(path)
+        path = FD.getExistingDirectory(caption=caption,
+            directory=root, options=QFileDialog.Option.ShowDirsOnly | QFileDialog.Option.ReadOnly)
+        if not path:
+            # the used pressed "cancel"
+            return
+        path = os.path.normpath(path)
+        sroot = paths.find_root(path)
 
-            root = os.path.normcase(os.path.normpath(root))
+        root = os.path.normcase(os.path.normpath(root))
 
-            if not sroot:
-                qtlib.WarningMsgBox(_('Cannot add subrepository'),
-                    _('%s is not a valid repository') % path,
-                    parent=self)
+        if not sroot:
+            qtlib.WarningMsgBox(_('Cannot add subrepository'),
+                _('%s is not a valid repository') % path,
+                parent=self)
+            return
+        if not os.path.isdir(sroot):
+            qtlib.WarningMsgBox(_('Cannot add subrepository'),
+                _('"%s" is not a folder') % sroot,
+                parent=self)
+            return
+        if os.path.normcase(sroot) == root:
+            qtlib.WarningMsgBox(_('Cannot add subrepository'),
+                _('A repository cannot be added as a subrepo of itself'),
+                parent=self)
+            return
+        if root != paths.find_root(os.path.dirname(os.path.normcase(path))):
+            qtlib.WarningMsgBox(_('Cannot add subrepository'),
+                _('The selected folder:<br><br>%s<br><br>'
+                'is not inside the target repository.<br><br>'
+                'This may be allowed but is greatly discouraged.<br>'
+                'If you want to add a non trivial subrepository mapping '
+                'you must manually edit the <i>.hgsub</i> file') % root, parent=self)
+            return
+        # The selected path is the root of a repository that is inside
+        # the selected repository
+
+        # Use forward slashes for relative subrepo root paths
+        srelroot = util.pconvert(hglib.fromunicode(sroot[len(root)+1:]))
+
+        # Is is already on the selected repository substate list?
+        try:
+            repo = hg.repository(hglib.loadui(),
+                                 hglib.fromunicode(root))
+        except:
+            qtlib.WarningMsgBox(_('Cannot open repository'),
+                _('The selected repository:<br><br>%s<br><br>'
+                'cannot be open!') % root, parent=self)
+            return
+
+        if srelroot in repo[b'.'].substate:
+            qtlib.WarningMsgBox(_('Subrepository already exists'),
+                _('The selected repository:<br><br>%s<br><br>'
+                'is already a subrepository of:<br><br>%s<br><br>'
+                'as: "%s"') % (sroot, root, hglib.tounicode(srelroot)),
+                                parent=self)
+            return
+        # Read the current .hgsub file contents
+        lines = []
+        hasHgsub = os.path.exists(repo.wjoin(b'.hgsub'))
+        if hasHgsub:
+            try:
+                with repo.wvfs(b'.hgsub', b'r') as fsub:
+                    lines = fsub.readlines()
+            except:
+                qtlib.WarningMsgBox(
+                    _('Failed to add subrepository'),
+                    _('Cannot open the .hgsub file in:<br><br>%s') \
+                    % root, parent=self)
                 return
-            elif not os.path.isdir(sroot):
-                qtlib.WarningMsgBox(_('Cannot add subrepository'),
-                    _('"%s" is not a folder') % sroot,
-                    parent=self)
+
+        # Make sure that the selected subrepo (or one of its
+        # subrepos!) is not already on the .hgsub file
+        linesep = b''
+        # On Windows case is unimportant, while on posix it is
+        srelrootnormcase = os.path.normcase(srelroot)
+        for line in lines:
+            spath = line.split(b"=")[0].strip()
+            if not spath:
+                continue
+            uline = hglib.tounicode(line)
+            if not linesep:
+                linesep = hglib.fromunicode(hglib.getLineSeparator(uline))
+            spath = util.pconvert(spath)
+            if os.path.normcase(spath) == srelrootnormcase:
+                qtlib.WarningMsgBox(
+                    _('Failed to add repository'),
+                    _('The .hgsub file already contains the '
+                    'line:<br><br>%s') % uline, parent=self)
                 return
-            elif os.path.normcase(sroot) == root:
-                qtlib.WarningMsgBox(_('Cannot add subrepository'),
-                    _('A repository cannot be added as a subrepo of itself'),
-                    parent=self)
-                return
-            elif root != paths.find_root(os.path.dirname(os.path.normcase(path))):
-                qtlib.WarningMsgBox(_('Cannot add subrepository'),
-                    _('The selected folder:<br><br>%s<br><br>'
-                    'is not inside the target repository.<br><br>'
-                    'This may be allowed but is greatly discouraged.<br>'
-                    'If you want to add a non trivial subrepository mapping '
-                    'you must manually edit the <i>.hgsub</i> file') % root, parent=self)
-                return
-            else:
-                # The selected path is the root of a repository that is inside
-                # the selected repository
+        if not linesep:
+            linesep = pycompat.oslinesep
 
-                # Use forward slashes for relative subrepo root paths
-                srelroot = sroot[len(root)+1:]
-                srelroot = util.pconvert(srelroot)
+        # Append the new subrepo to the end of the .hgsub file
+        lines.append(b'%s = %s' % (srelroot, srelroot))
+        lines = [line.strip(linesep) for line in lines]
 
-                # Is is already on the selected repository substate list?
-                try:
-                    repo = hg.repository(hglib.loadui(),
-                                         hglib.fromunicode(root))
-                except:
-                    qtlib.WarningMsgBox(_('Cannot open repository'),
-                        _('The selected repository:<br><br>%s<br><br>'
-                        'cannot be open!') % root, parent=self)
-                    return
+        # and update the .hgsub file
+        try:
+            with repo.wvfs(b'.hgsub', b'w') as fsub:
+                fsub.write(linesep.join(lines) + linesep)
 
-                if hglib.fromunicode(srelroot) in repo[b'.'].substate:
-                    qtlib.WarningMsgBox(_('Subrepository already exists'),
-                        _('The selected repository:<br><br>%s<br><br>'
-                        'is already a subrepository of:<br><br>%s<br><br>'
-                        'as: "%s"') % (sroot, root, srelroot), parent=self)
-                    return
-                else:
-                    # Read the current .hgsub file contents
-                    lines = []
-                    hasHgsub = os.path.exists(repo.wjoin(b'.hgsub'))
-                    if hasHgsub:
-                        try:
-                            with repo.wvfs(b'.hgsub', b'r') as fsub:
-                                lines = fsub.readlines()
-                        except:
-                            qtlib.WarningMsgBox(
-                                _('Failed to add subrepository'),
-                                _('Cannot open the .hgsub file in:<br><br>%s') \
-                                % root, parent=self)
-                            return
-
-                    # Make sure that the selected subrepo (or one of its
-                    # subrepos!) is not already on the .hgsub file
-                    linesep = ''
-                    # On Windows case is unimportant, while on posix it is
-                    srelrootnormcase = os.path.normcase(srelroot)
-                    for line in lines:
-                        line = hglib.tounicode(line)
-                        spath = line.split("=")[0].strip()
-                        if not spath:
-                            continue
-                        if not linesep:
-                            linesep = hglib.getLineSeparator(line)
-                        spath = util.pconvert(spath)
-                        if os.path.normcase(spath) == srelrootnormcase:
-                            qtlib.WarningMsgBox(
-                                _('Failed to add repository'),
-                                _('The .hgsub file already contains the '
-                                'line:<br><br>%s') % line, parent=self)
-                            return
-                    if not linesep:
-                        linesep = os.linesep
-
-                    # Append the new subrepo to the end of the .hgsub file
-                    lines.append(hglib.fromunicode('%s = %s'
-                                                   % (srelroot, srelroot)))
-                    lines = [line.strip(linesep) for line in lines]
-
-                    # and update the .hgsub file
-                    try:
-                        with repo.wvfs(b'.hgsub', b'w') as fsub:
-                            fsub.write(linesep.join(lines) + linesep)
-
-                        if not hasHgsub:
-                            commands.add(hglib.loadui(),
-                                         repo, repo.wjoin(b'.hgsub'))
-                        qtlib.InfoMsgBox(
-                            _('Subrepo added to .hgsub file'),
-                            _('The selected subrepo:<br><br><i>%s</i><br><br>'
-                            'has been added to the .hgsub file of the repository:<br><br><i>%s</i><br><br>'
-                            'Remember that in order to finish adding the '
-                            'subrepo <i>you must still <u>commit</u></i> the '
-                            'changes to the .hgsub file in order to confirm '
-                            'the addition of the subrepo.') \
-                            % (srelroot, root), parent=self)
-                    except:
-                        qtlib.WarningMsgBox(
-                            _('Failed to add repository'),
-                            _('Cannot update the .hgsub file in:<br><br>%s') \
-                            % root, parent=self)
-                return
+            if not hasHgsub:
+                commands.add(hglib.loadui(),
+                             repo, repo.wjoin(b'.hgsub'))
+            qtlib.InfoMsgBox(
+                _('Subrepo added to .hgsub file'),
+                _('The selected subrepo:<br><br><i>%s</i><br><br>'
+                'has been added to the .hgsub file of the repository:<br><br><i>%s</i><br><br>'
+                'Remember that in order to finish adding the '
+                'subrepo <i>you must still <u>commit</u></i> the '
+                'changes to the .hgsub file in order to confirm '
+                'the addition of the subrepo.') \
+                % (hglib.tounicode(srelroot), root), parent=self)
+        except:
+            qtlib.WarningMsgBox(
+                _('Failed to add repository'),
+                _('Cannot update the .hgsub file in:<br><br>%s') \
+                % root, parent=self)
 
     def removeSubrepo(self):
         'menu action handler for removing an existing subrepository'
-        model = self.tview.model()
+        model = cast(repotreemodel.RepoTreeModel, self.tview.model())
         index = self.tview.currentIndex()
         assert model is not None
-        path = model.repoRoot(index)
-        root = model.repoRoot(index.parent())
-        relsubpath = os.path.normcase(os.path.normpath(path[1+len(root):]))
-        hgsubfilename = os.path.join(root, '.hgsub')
+        upath = model.repoRoot(index)
+        uroot = model.repoRoot(index.parent())
+        urelsubpath = os.path.normcase(os.path.normpath(upath[1+len(uroot):]))
+        relsubpath = hglib.fromunicode(urelsubpath)
+        hgsubfilename = os.path.join(hglib.fromunicode(uroot), b'.hgsub')
 
         try:
             hgsub = []
             found = False
-            with open(hgsubfilename, 'r') as f:
+            with open(hgsubfilename, 'rb') as f:
                 for line in f:
                     spath = os.path.normcase(
                         os.path.normpath(
-                            line.split('=')[0].strip()))
+                            line.split(b'=')[0].strip()))
                     if spath != relsubpath:
                         hgsub.append(line)
                     else:
@@ -753,12 +774,12 @@ class RepoRegistryView(QDockWidget):
         choices = (_('&Yes'), _('&No'))
         answer = qtlib.CustomPrompt(_('Remove the selected repository?'),
             _('Do you really want to remove the repository "<i>%s</i>" '
-              'from its parent repository "<i>%s</i>"') % (relsubpath, root),
-            self, choices=choices, default=choices[0]).run()
+              'from its parent repository "<i>%s</i>"') % (urelsubpath, uroot),
+            self, choices=choices, default=0).run()
         if answer != 0:
             return
         try:
-            with open(hgsubfilename, 'w') as f:
+            with open(hgsubfilename, 'wb') as f:
                 f.writelines(hgsub)
             qtlib.InfoMsgBox(_('Subrepository removed from .hgsub'),
                 _('The selected subrepository has been removed '
@@ -777,7 +798,7 @@ class RepoRegistryView(QDockWidget):
         sd = settings.SettingsDialog(configrepo=True, focus='web.name',
                                      parent=self, root=root)
         sd.finished.connect(sd.deleteLater)
-        sd.exec_()
+        sd.exec()
 
     def openAll(self):
         index = self.tview.currentIndex()
@@ -846,8 +867,8 @@ class RepoRegistryView(QDockWidget):
         if 'remove' not in item.menulist():  # check capability
             return
         if not item.okToDelete():
-            labels = [(QMessageBox.Yes, _('&Delete')),
-                      (QMessageBox.No, _('Cancel'))]
+            labels = [(QMessageBox.StandardButton.Yes, _('&Delete')),
+                      (QMessageBox.StandardButton.No, _('Cancel'))]
             if not qtlib.QuestionMsgBox(_('Confirm Delete'),
                                         _("Delete Group '%s' and all its "
                                           "entries?") % item.name,
@@ -924,8 +945,7 @@ class RepoRegistryView(QDockWidget):
                 % (root, "<br>".join(invalidpaths)), parent=self)
 
     @pyqtSlot(str)
-    def scanRepo(self, uroot):
-        uroot = pycompat.unicode(uroot)
+    def scanRepo(self, uroot: str):
         m = self.tview.model()
         assert m is not None
         index = m.indexFromRepoRoot(uroot)
