@@ -6,6 +6,12 @@
 # GNU General Public License version 2 or any later version.
 
 import re
+from typing import (
+    List,
+    Optional,
+)
+
+hgversion: Optional[bytes] = None
 
 try:
     try:
@@ -17,11 +23,11 @@ try:
         from mercurial import version  # pytype: disable=import-error
         hgversion = version.get_version()
 except ImportError:
-    hgversion = None
+    pass
 
-testedwith = b'5.9 6.0 6.1 6.2'
+testedwith = b'5.9 6.0 6.1 6.2 6.3 6.4 6.5'
 
-def _splitversion(v):
+def _splitversion(v: bytes) -> Optional[List[bytes]]:
     """Extract (major, minor) version components as bytes, or None"""
     v = v.split(b'+')[0]
     if not v or v == b'unknown' or len(v) >= 12:
@@ -32,7 +38,7 @@ def _splitversion(v):
         return
     return vers
 
-def checkhgversion(v):
+def checkhgversion(v: bytes) -> Optional[bytes]:
     """range check the Mercurial version"""
     reqvers = testedwith.split()
     vers = _splitversion(v)
@@ -43,7 +49,7 @@ def checkhgversion(v):
     return (b'This version of TortoiseHg requires Mercurial version %s.n to '
             b'%s.n, but found %s') % (reqvers[0], reqvers[-1], v)
 
-def checkminhgversion(v):
+def checkminhgversion(v: bytes) -> Optional[bytes]:
     """Check if the given Mercurial version is not lower than the minimum
     supported version
 
